@@ -25,6 +25,7 @@ module Basement.Block.Base
     , mutableLength
     , mutableLengthBytes
     -- * Other methods
+    , empty
     , mutableEmpty
     , new
     , newPinned
@@ -55,13 +56,17 @@ import           Basement.PrimType
 data Block ty = Block ByteArray#
     deriving (Typeable)
 
+unsafeBlockPtr :: Block ty -> Ptr ty
+unsafeBlockPtr (Block arrBa) = Ptr (byteArrayContents# arrBa)
+{-# INLINE unsafeBlockPtr #-}
+
 instance Data ty => Data (Block ty) where
     dataTypeOf _ = blockType
     toConstr _   = error "toConstr"
     gunfold _ _  = error "gunfold"
 
 blockType :: DataType
-blockType = mkNoRepType "Foundation.Block"
+blockType = mkNoRepType "Basement.Block"
 
 instance NormalForm (Block ty) where
     toNormalForm (Block !_) = ()
@@ -147,9 +152,11 @@ internalFromList l = runST $ do
     ma <- new (CountOf len)
     iter azero l $ \i x -> unsafeWrite ma i x
     unsafeFreeze ma
-  where len = Data.List.length l
-        iter _  []     _ = return ()
-        iter !i (x:xs) z = z i x >> iter (i+1) xs z
+  where
+    !len = Data.List.length l
+
+    iter _  []     _ = return ()
+    iter !i (x:xs) z = z i x >> iter (i+1) xs z
 
 -- | transform a block to a list.
 internalToList :: forall ty . PrimType ty => Block ty -> [ty]
@@ -395,8 +402,8 @@ withPtr :: PrimMonad prim
 withPtr x@(Block ba) f
     | isPinned x == Pinned = f (Ptr (byteArrayContents# ba)) <* touch x
     | otherwise            = do
-        arr@(Block arrBa) <- makeTrampoline
-        f (Ptr (byteArrayContents# arrBa)) <* touch arr
+        arr <- makeTrampoline
+        f (unsafeBlockPtr arr) <* touch arr
   where
     makeTrampoline = do
         trampoline <- unsafeNew Pinned (lengthBytes x)
@@ -478,5 +485,5 @@ withMutablePtrHint skipCopy skipCopyBack mb f
   where
     vecSz = mutableLengthBytes mb
     callWithPtr pinnedMb = do
-        b@(Block ba) <- unsafeFreeze pinnedMb
-        f (Ptr (byteArrayContents# ba)) <* touch b
+        b <- unsafeFreeze pinnedMb
+        f (unsafeBlockPtr b) <* touch b
