@@ -751,8 +751,8 @@ builderBuild :: PrimMonad m => Int -> Builder (Array ty) (MArray ty) ty m err ()
 builderBuild sizeChunksI ab
     | sizeChunksI <= 0 = builderBuild 64 ab
     | otherwise        = do
-        first         <- new sizeChunks
-        ((), (i, st, e)) <- runState (runBuilder ab) (Offset 0, BuildingState [] (CountOf 0) first sizeChunks, Nothing)
+        first      <- new sizeChunks
+        (i, st, e) <- snd <$> runState (runBuilder ab) (Offset 0, BuildingState [] (CountOf 0) first sizeChunks, Nothing)
         case e of
           Just err -> pure (Left err)
           Nothing -> do
