@@ -34,8 +34,7 @@ import           GHC.STRef
 import           GHC.IORef
 import           GHC.IO
 import           GHC.Prim
-import           Basement.Compat.Base (Exception, (.), ($), Applicative)
-import           Basement.Compat.AMP
+import           Basement.Compat.Base (Exception, (.), ($), Applicative, Monad)
 
 -- | Primitive monad that can handle mutation.
 --
@@ -122,7 +121,7 @@ primTouch x = unsafePrimFromIO $ primitive $ \s -> case touch# x s of { s2 -> (#
 -- | Monad that can represent failure
 --
 -- Similar to MonadFail but with a parametrized Failure linked to the Monad
-class AMPMonad m => MonadFailure m where
+class Monad m => MonadFailure m where
     -- | The associated type with the MonadFailure, representing what
     -- failure can be encoded in this monad
     type Failure m
